require 'pathname'

def fix_regex(regex_string)
  regex_string.sub('~', '/').reverse.sub('~', '/').reverse
end

def generate_version_class(client, regex)
  file_name = "regex_version_#{client}.rb"
  code = <<-CODE
# Generated by generate_regex_classes.rb
class SameSiteUaChecker
  class RegexVersion#{client.capitalize} < RegexVersionAbstract
    def regex_version
      #{fix_regex(regex)}
    end
  end
end
CODE
  File.open(File.join(__dir__, '..', 'lib', 'same_site_ua_checker', file_name), 'w') { |f| f.puts(code) }
end

def generate_name_class(case_name, regexs)
  parts = case_name.split('-')
  file_name = "regex_list_#{parts.join('_')}.rb"
  code = <<-CODE
# Generated by generate_regex_classes.rb
class SameSiteUaChecker
  class RegexList#{parts.map(&:capitalize).join} < RegexListAbstract
    def regex_list
      [
        #{regexs.map { |regex| fix_regex(regex) }.join(', ')}
      ]
    end
  end
end
CODE
  File.open(File.join(__dir__, '..', 'lib', 'same_site_ua_checker', file_name), 'w') { |f| f.puts(code) }
end

Dir[File.join(__dir__, '..', '..', '..', 'regex', 'version-match', '*')].each do |path|
  f = File.open(path)
  regex = f.read
  f.close
  client = Pathname.new(path).basename.to_s

  generate_version_class(client, regex)
end

Dir[File.join(__dir__, '..', '..', '..', 'regex', 'check', '*')].each do |path|
  f = File.open(path)
  lines = f.readlines.map(&:strip)
  f.close
  client = Pathname.new(path).basename.to_s

  generate_name_class(client, lines)
end
