require_relative 'same_site_ua_checker/version'

require_relative 'same_site_ua_checker/regex_version_abstract'
require_relative 'same_site_ua_checker/regex_version_chrome'
require_relative 'same_site_ua_checker/regex_version_ucbrowser'

require_relative 'same_site_ua_checker/regex_list_abstract'
require_relative 'same_site_ua_checker/regex_list_same_site_none_bug'
require_relative 'same_site_ua_checker/regex_list_same_site_not_supported'
require_relative 'same_site_ua_checker/regex_list_same_site_strict_bug'

class SameSiteUaChecker
  # @param [String] user_agent_string
  # @return [Boolean] Browser has bug and should NOT receive SameSite=None cookie attribute
  def self.call(user_agent_string)
    new.check(user_agent_string)
  end

  def check(user_agent_string)
    raise ArgumentError, 'Only string argument is allowed' unless user_agent_string.is_a?(String)

    check_same_site_not_supported(user_agent_string) ||
      check_same_site_strict_bug(user_agent_string) ||
      check_same_site_none_bug(user_agent_string)
  end

  private

  def check_same_site_not_supported(user_agent_string)
    RegexListSameSiteNotSupported.new.check(user_agent_string)
  end

  def check_same_site_none_bug(user_agent_string)
    # Additional check for chrome browser version
    is_chrome_based_browser = RegexListSameSiteNoneBug.new.check(user_agent_string)

    return false unless is_chrome_based_browser

    ucbrowser_version = RegexVersionUcbrowser.new.detect(user_agent_string)
    return !ucbrowser_version.at_least?(12, 13, 2)  if ucbrowser_version.major != 0

    chrome_browser_version = RegexVersionChrome.new.detect(user_agent_string)
    return chrome_browser_version.major_between_inc?(51, 66) if chrome_browser_version.major != 0

    false
  end

  def check_same_site_strict_bug(user_agent_string)
    is_chrome_based_browser = RegexListSameSiteNoneBug.new.check(user_agent_string)
    return false if is_chrome_based_browser

    RegexListSameSiteStrictBug.new.check(user_agent_string)
  end
end
